from OFS import Folder
from Globals import HTMLFile
from AccessControl import getSecurityManager, Permissions


manage_addImagePagerFolderForm=HTMLFile('imagePagerFolderAddForm', globals())

def manage_addImagePagerFolder(self, id, title,
                           thumbnailsPerPage=10, REQUEST=None):
    """Add a new ImagePagerFolder object with id *id*, title *title*.

    """
    id=str(id)
    title=str(title)

    ob=ImagePagerFolder()
    ob.id=id
    ob.title = title

    ob.manage_addProperty('thumbnailsPerPage', int(thumbnailsPerPage), 'int')

    self._setObject(id, ob)
    ob=self._getOb(id)

    checkPermission=getSecurityManager().checkPermission

    index_html = HTMLFile('index_html', globals())
    text = index_html.read_raw()
    #text = str(dir(index_html))

    if not checkPermission('Add Folders', ob):
        raise 'Unauthorized', (
               'You are not authorized to add ImagePagerFolders.'
               )
    ob.manage_addDTMLMethod(id='index_html', title='')
    idx = ob.index_html
    idx.manage_edit(text, title='')

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


class ImagePagerFolder(Folder.Folder):
    """Folder which automatically thumbnails and links images.

    """

    meta_type="ImagePagerFolder"
    manage_thumbnailsForm=HTMLFile('thumbnailsForm', globals())

    manage_options=(Folder.Folder.manage_options+
                    ({ "label": "Thumbnails",
                       "action": "manage_thumbnailsForm"},)
                    )

    def manage_setThumbnailsPerPage(self, thumbnailsPerPage=10, REQUEST=None):
        """Set the number of thumbnails show per page."""

        checkPermission=getSecurityManager().checkPermission
        if not checkPermission('Manage properties', self):
            raise 'Unauthorized', ('You are not authorized to change that setting')

        self.thumbnailsPerPage = int(thumbnailsPerPage)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect('manage_main')


    def manage_makeThumbnails(self, folderNames=[], size=128, REQUEST=None):
        """Make thumbnail images of all images in folders in folderNames
        in self. If folderNames is an empty list, do all folders
        which start with the word 'images' or if neither of those
        finds any folders try using self.

        """

        checkPermission=getSecurityManager().checkPermission
        if not checkPermission('Add Folders', self):
            raise 'Unauthorized', ('You are not authorized to make thumbnails')

        if not folderNames:
            folders = self.objectValues('Folder')
            for folder in folders:
                id = folder.getId()
                if not id.startswith('images'):
                    folders.remove(folder)
        else:
            folders = []
            for name in folderNames:
                folders.append(getattr(self, name))

        if not folders:
            folders = [self]

        for folder in folders:
            images = folder.objectValues('Image')
            for image in images:
                if hasattr(image, 'original_id'):
                    continue
                original_id = image.getId()
                try:
                    self._makeThumbnail(folder, original_id, size=size)
                except IOError:
                    pass

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect('manage_main')


    def thumbnailsList(self):
        folders = self.objectValues('Folder')

        imageFolders = []
        for folder in folders:
            id = folder.getId()
            if id.startswith('images'):
                imageFolders.append(folder)
        if not imageFolders:
            imageFolders = [self]

        thumblist = []
        for folder in imageFolders:
            images = folder.objectValues()

            for image in images:
                id = image.getId()
                if id.find("_t") >= 0:
                    thumblist.append(image)

        return thumblist


    def getOriginalImage(self, image):
        """Get original image from thumbnail (using original_id)"""

        f = image.aq_parent
        o = image.original_id

        original = getattr(f, o)

        return original


    def getOriginalURL(self, image):
        """Get original image URL from thumbnail"""

        f = image.aq_parent
        o = image.original_id

        original = getattr(f, o)

        return original.absolute_url()


    def simplePager(self, page=0, REQUEST=None):
	"""Return links to thumbnail pages."""
        req = REQUEST
        url = req.URL
        seq = self.thumbnailsList()

        if req.has_key('start'):
            start = getattr(req, 'start')
        else:
            start = 0

        size = self.thumbnailsPerPage
        starts = range(1, len(seq)+1, size)

        if len(starts) < 2:
            return ""

        printed = ""
        p = 1
        for s in starts:
            printed += '<a href="%s?start=%s">%s</a>\n' % (url, s, p)
            p += 1

        return printed



    def _makeThumbnail(container, self, original_id, newExt='_t', size=128):
        """
        Makes a thumbnail image given an image Id when called on a Zope
        folder.

        The thumbnail is a Zope image object that is a small JPG
        representation of the original image. The thumbnail has a
        'original_id' property set to the id of the full size image
        object.
        """

        from PIL import Image
        from cStringIO import StringIO
        import os.path

        # create a thumbnail image file
        original_image = getattr(self, original_id)
        original_file = StringIO(str(original_image.data))
        image = Image.open(original_file)
        image = image.convert('RGB')
        image.thumbnail((int(size),int(size)))
        thumbnail_file = StringIO()
        image.save(thumbnail_file, "JPEG") 
        thumbnail_file.seek(0)

        # create an id for the thumbnail
        path, ext = os.path.splitext(original_id)
        thumbnail_id = path + newExt

        # if there's and old thumbnail, delete it
        if thumbnail_id in self.objectIds():
            self.manage_delObjects(thumbnail_id)

        if original_image.title:
            title = original_image.title
        else:
            title = 'thumbnail image'
 
	
        # create the Zope image object
        self.manage_addProduct['OFSP'].manage_addImage(thumbnail_id,
                                                   thumbnail_file, title)

        thumbnail_image = getattr(self, thumbnail_id)

        # set the 'originial_id' property
        thumbnail_image.manage_addProperty('original_id', 
                                             original_id, 'string')

        #return 'size='+`size`+' path='+path+' newExt='+newExt+' id='+thumbnail_id
